# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.5.3
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x02M\
\x00\
\x00\x0d\x94x\xda\xed\x97Mo\xe20\x10\x86\xef\xfe\x15\
\xfb\x03\xac*\x9e|\x90V\xca\xa1\x14\xba\xad\xb6\x8bV\
b\xd5\x1e\x10\x07o\x18 j\xb0+\xc7\xb4\xea\xfe\xfa\
\x9d|\x00\x86\xa4jU\xb1{X!\xc5\x87\xcck;\
\xef<3r\x92\xc9\x95\xce\xb5\x19\xce\xe7\x98\xda\xe2b\
\x90\x15\xf2W\x8e\xb3)\xab\xc2I\x18\xf1\xea\xaao/\
Wz\xadl\xe21gMu\xa7\xac\x91\x85\xdd\xc8g\
Q\xb8\x8d5\x93\x04\xbbU\x16U\x91\xd9\xd7\xed,'\
\xd6\xcc\x02\xc6&{vn\x95Lm\xf6\x8cdg)\
\xd5\x02\xc7\x98S<\xd3\xaavg\xcd\x1a\x1b\xa3B\x00\
\x17B\xd0\xf0\xf6\xbd\x9ey\x10\xee\xf9\x85\xb6_qh\
\x17\xd8P\x95\x1c\x92\xb9\xcc\x0bl{o9\xf76\xce\
\x8b\x8b\xfe\xdaZ\xad\xa6\xac/\xd3\xc7\x85\xa1\xf9\xb3\xcb\
\xdc\xa2Q\xd2b\x22\x22\x9f\x039\x85\xd0s\xf4\x916\
+\x99'\x10\xf4x3\xd8\x00Smd\x99\xe9\xb5N\
\xd7E\x12Qf\xbd\x80\x83\xef;\xd2\x8d~F\xe3J\
\xd7\xda`\xf3\xc8\x0a[\xb7\xb6\x81Z3\x83\x90\x0b?\
v\xe4\xbbL=&\x01-\x84\x98\x8b8t\x94\x11.\
d\xb5\x10D\xcc\xa3\x98\xc7\xfe\x9e\xb8&\x82e\x16\x11\
\x15!\xe2\x9e\xab\xd5\x09\xfa!\xf7c\x1e\x08G\xf9\xa1\
\x09b\xb9\xa5\x7f^9=\x8f\x1c\xf1>#\x11g\x89\
\x08C\x1e\x93\x1e\xc3\x8e\xf2\x95^=\xe5\xb8Be\xa5\
y\xed\x86\x0d=\xee{\xdc\x876\xe9\x008\xb9\xa4\x1a\
\xfc\x1b\xcc\xa5\x1c\x95\xf9\xf5\x0e1\x03ECj\x89\xc0\
?\x22f\x08\xcb\x06\xab\xc61H\xdf\xa0\x9c\xa1y\x03\
1m\x06\x81GCt\xf43\x90\x07 ,\x10\x9d\xfa\
\xf9\x83\x94'\xbb\x13\xaf\x937\xf54P\xd3\x80\xdf\xc5\
\xdb)\xc6\x89\xf7;\xbc\xb7o\x92O\x1c\xd4\x9d0\xff\
#\xce@\xbc\x9aq\x0c\xd4?\xb5\xcem\xf6\xf4\x99\x13\
\xe4\xf4F\xfc8\xe6\xfb\x0c_\xde`\xec`l3v\
\x8a}b\xfc\x0e\xe3\x87L\xcd\xf4\xcb\xe9l\xfeK\x94\
\xbf\xa2BzN\xf3\xf31N\x97\xf4\x81\x97\xf4\x0d\xe2\
o\xfcr\x97-\x96\x96\x8d\xe4a\xa4X\xd2\x8bs\xac\
\x8d\xa55\xeb\x95\xaa\xff\x09\xd8\xe4\xdb`8ei\xf3\
U\x9f\x04\x14y\xf8>e5\x9c]u\xf6\x8a\xd6\x88\
9v\xc6w\xd6w\xb9f\xaa\xbd\xa1S\xe8\xad\x5co\
\xd9\xa18\x9b\xba\x05\xfb\x03^\xd9Vy\
\x00\x00\x02&\
\x00\
\x00\x0d~x\xda\xed\x97]o\xda0\x14\x86\xef\xfd+\
\xf6\x03\xac*v\xe2\x10*\xf9\xa2\x14\xbaV\xdb\xd0$\
\xaa\xf6\x02q\xe1\x85\x03D\x04\xbbr\x9cV\xdd\xaf\xdf\
!\x1f`\x0a\x93\xa8\xbaJ\xbb\x88\x94\x5c\xe4\xbc\xc7\xce\
{\x1e[\xfe\x98^\x9b\xdc\xd8\xd1b\x01\xa9+.\x87\
Y\xa1~\xe50\x9f\x91*,EL\xab\xa7\xfe\xbc\xda\
\x98R;\x19\x10\xafM\xf5\xa5\x9dU\x85k\xe5\x8bX\
\xecbM\x12#w\xda\x81.2\xf7\xba\xcb\xf2bM\
\x16'dz`\xe7N\xab\xd4e\xcf\x80vVJ/\
a\x029\xc63\xa3kw\xce\x96\xd0\x18e\x8cS\xc6\
\x18\xbe\xc1\xa1\xd7\x8b\x80\x8b\x03\xbf\xfc\xd8/{k\x97\
\x93\x91\xder\x90\x0b\x95\x17p\xec\xfd\xc8y\xd0:/\
.\x07\xa5sF\xcf\xc8@\xa5\xeb\xa5\xc5\xfc\xf9U\xee\
\xc0j\xe5@\x86\x01Mz\xe81\xf6\xd4\xb1\xb1\x1b\x95\
\xcb\xa8OEDE\x9f\x0c!5Vm\x8b\xbc1i\
Y\xc8\x18\x8b\xeaE\x94\x87\xa1'\xdd\x9ag\xb0\xbet\
c,4\x7f\xab\x88\x9d\xd6Z\x9e\x92m\xe5\xb8\x8f)\
=O\xfe\x9e\xe9\xb5\xe4\x18\x15!\xe5\x91\xdfp\x0cK\
U5\xe4,\xa1qB\x93C\xb1Dx\xb9\xe4Q\xbc\
\xad\x8d\x06\xbeVW\xc7\x05\xa7\xcd\xeb\x89?\x0d\x22\xdc\
\xf6\x1a\xf6+\xb3\xfd\xd8\x13\x1f2\x14a.\x99\x104\
A=\xe1{\xc6\xd7f\xf3\x94\xc3\x06\xb4S\xf6\xf5\xfd\
\xa89E\xa7\x22\xe8P\x9f\x81\xfa\x16\xd4\x1c\xeci\xc6\
;\x90\xddl\xfe\x07\x88\xa7\xfb\xd5\xee$\xec\x16i7\
\x9f?\x02{\xb7\x85\xbc{\xd98I\xf3C\xa0O\x97\
v\x16i\xc4\xcc\x92\x84\xf6\xc4\xffK\xfa\xde\x98\xdce\
O\xdd\xd2\xf1\x89\x8c\x1f2x\xf9\xcbD\x164Lh\
\xc4\x8e\x01\xf3\x1e\xc5Y\x1e\xf2\x0e\xf0\x19\x80\x1f3=\
7/\xdd\x8a\xfc\x19\x88\xbf\x82\x06\xfcUs\xd7\x98\xa4\
+<\xd1\xc9\x81\x05\xf8\x0d_\x86\xca\xae\xc9X\xbd\x09\
\x14+\xdc)'\xc6:lQnt}\x01 \xd3o\
\xc3\xd1\x8c\xa4\xcd\x11^F\x18y\xfc1#5\x9d\xfd\
\xd8\xec\xc7\xabQr\xc0\xa0_L\x1d\xdf\xbb>\x103\
}\xdca;\xcc;\xad\xea\xd2\x1f\x8fV\xf1:\xf5\xe5\
?T\xf7RF\
"

qt_resource_name = b"\
\x00\x06\
\x07\xae\xc3\xc3\
\x00t\
\x00h\x00e\x00m\x00e\x00s\
\x00\x06\
\x06\x98\xbd\x05\
\x00b\
\x00r\x00e\x00e\x00z\x00e\
\x00\x0b\
\x00\xd0\xbf+\
\x00b\
\x00r\x00e\x00e\x00z\x00e\x00-\x00d\x00a\x00r\x00k\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00$\x00\x01\x00\x00\x00\x01\x00\x00\x02Q\
\x00\x00\x01\x8b\x0an\xba\x97\
\x00\x00\x00\x12\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x8b\x0an\xba\x97\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
